/* $Id: joblist.hh,v 1.5 2001/11/19 00:04:51 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   ` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Interface to the GtkCList of running jobs (downloads etc),
  GUI::window.jobs, i.e. the list at the bottom of the jigdo window.

*/

#ifndef JOBLIST_HH
#define JOBLIST_HH

#include <string>
#include <vector>
namespace std { }
using namespace std;

#include <debug.hh>
#include <download.hh>
#include <gtk-gui.hh>
//______________________________________________________________________

class JobVector;

/** One "job", e.g. file download, scanning through files, or image
    download (which has many file downloads as children). */
class Job {
  friend class JobVector; // Overwrites listVal and rowNrVal on insert()
public:
  // Job is stopped immediately after creation and after error/abort
  enum State { stopped, running, paused };

  Job() : stateVal(stopped) { }
  virtual ~Job() = 0;
  /** Start/restart this job. When called, must also create GUI
      elements for this Job in row number row() of GtkCList list(). Is
      called after Job is added to the list, and when it is restarted
      (if at all). Child classes should provide an implementation */
  virtual void run() { }
  /// Get current state of job.
  inline State state() const;

  /// List to use when updating the GUI elements
  inline GtkCList* list() const;
  /// Row number in list. May change during collapse()
  inline size_t row() const;

protected:
  inline void setState(const State s);

private:
  GtkCList* listVal;
  size_t rowVal;
  State stateVal;
};
//______________________________________________________________________

/** A bit like a vector<Job*>, but uses the GtkCList for storing the
    elements. Big difference from vector<>: When deleting elements,
    the elements after the deleted one are not copied by one position
    immediately. Instead, such copying only happens during collapse().
    Reason: User should not be annoyed by rows becoming non-selectable
    because they move around the list so quickly. OTOH, insertion
    happens immediately. */
class JobVector {
public:
  // Assumed number of columns in job display (icon, progress bar, URL)
  static const int COLUMN_PROGRESS = 0;
  static const int COLUMN_OBJECT = 1;
  static const int NR_OF_COLUMNS = 2;
  static const int PIXMAP_SPACING = 0; // In pixmap+text GtkCList cells

  typedef unsigned size_type; ///
  inline JobVector();
  /// Any Jobs still in the list are deleted
  inline ~JobVector();

  /** Size *includes* elements that have been delete()d, but not yet
      collapse()d away. */
  inline size_type size() const;
  /// Number of non-deleted entries (always <= size())
  inline size_type entryCount() const;
  inline bool empty() const;

  /// Use this instead of operator[]
  inline Job* get(size_type n);
  /// Simply overwrites pointer, will not delete the old entry
  inline void set(size_type n, Job* j);
  /** Makes row blank so that collapse() will eventually remove it.
      The Job is deleted. */
  void erase(size_type n);
  /** Insert new Job before position n. Calls the Job's display(n)
      method so the Job can visualize itself. */
  inline void insert(size_type n, Job* j);

  /** This should be called at regular intervals, every second or so.
      Removes *all* deleted rows that are not currently visible to the
      user. Removes *first* of the deleted rows on screen. */
  inline void collapse();

  void freeze() { gtk_clist_freeze(list()); }
  void thaw() { gtk_clist_thaw(list()); }

  /** To be called sometime after gtk_init, but before any
      Job(Vector)s are used. */
  static void initAfterGtk();

  // Return pointer to appropriate "x% done" pixmap
  static GdkPixmap* progress(uint64 curAmount, uint64 totalAmount);

private:
  // We save the room for the pointer, since it always points to window.jobs
  // GtkCList* listVal;
  GtkCList* list() const { return GTK_CLIST(GUI::window.jobs); }
  static void deleteJob(void* j); // Just deletes the Job
  inline void collapse2(); // Actually performs collapsing of list

  /* Used by initAfterGtk(): Nr of pixmaps to subdivide the progress
     XPM into, filename to load from. */
  static const int PROGRESS_SUBDIV = 51;
  static const char* const PROGRESS_XPM = "progress051.xpm";
  static vector<GdkPixmap*> progressGfx;
  static gchar* noText[]; // For init'ing CList row

  unsigned entryCountVal;
};
//______________________________________________________________________

/// Global list of running jobs
namespace GUI {
  extern JobVector jobVector;
}
//______________________________________________________________________

/// Job: Download of a file.
class DownloadJob : public Job, private Download::Output {
public:
  DownloadJob(const string& uri, const string& dest);
  // Virtual methods from Job
  virtual ~DownloadJob();
  virtual void run();

  // Virtual methods from Download::Output
//   virtual void dataSize(uint64 n);
//  virtual void newData(void* data, size_t size, uint64 totalSize);
//   virtual void info(const string& message);
//   virtual void error(const string& message);

private:
  Download download;
  string destination;
  GtkWidget* progressBar;
};
//______________________________________________________________________

Job::State Job::state() const { return stateVal; }
GtkCList* Job::list() const { return listVal; }
size_t Job::row() const { return rowVal; }
void Job::setState(const State s) { stateVal = s; }
//________________________________________

JobVector::JobVector() : entryCountVal(0) {
  // Mustn't access GtkCList here because GTK+ is not initialized yet!
}
JobVector::~JobVector() {
  /* Don't do gtk_clist_clear() - the CList is auto-deleted because it
     is the child of a deleted window. */
}

JobVector::size_type JobVector::size() const { return list()->rows; }
JobVector::size_type JobVector::entryCount() const { return entryCountVal; }
bool JobVector::empty() const { return size() == 0; }

Job* JobVector::get(size_type n) {
  return static_cast<Job*>(gtk_clist_get_row_data(list(), n));
}
void JobVector::set(size_type n, Job* j) {
  Paranoid(j != 0);
  gtk_clist_set_row_data_full(list(), n, j, deleteJob);
  j->listVal = list();
  j->rowVal = n;
}
void JobVector::insert(size_type n, Job* j) {
  Paranoid(j != 0);
  gtk_clist_insert(list(), n, noText); // 0 => no text initially
  gtk_clist_set_row_data_full(list(), n, j, deleteJob);
  ++entryCountVal;
  j->listVal = list();
  j->rowVal = n;
  j->run();
}

void JobVector::collapse() {
  if (entryCountVal < size()) collapse2();
}

#endif
