/* $Id: serialize.hh,v 1.1 2001/09/05 21:47:12 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   ` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Convert objects into byte streams and vice versa

  Classes that support serialization should implement the indicated
  member functions; serialize(), unserialize() and serialSizeOf(). It
  is assumed that 'byte' has been typdef'd to an unsigned type which
  represents one byte.

  MyClass {
    template<class Iterator>
    inline Iterator serialize(Iterator i) const;
    template<class ConstIterator>
    inline ConstIterator unserialize(ConstIterator i);
    inline size_t serialSizeOf() const;
  };

*/

#ifndef SERIALIZE_HH
#define SERIALIZE_HH

#include <config.h> /* byte, size_t */
//______________________________________________________________________

/* Store serialized object via iterator. The iterator could be byte*
   or vector<byte>::iterator - anything in which you can store bytes.
   There must be enough room to store serialSizeOf() bytes,
   implementers of serialize() need not check this. */
template<class Object, class Iterator>
inline Iterator serialize(const Object& o, Iterator i) {
  return o.serialize(i);
}

/* Assign the contents of the byte stream pointed to by i to the object.
   template<class Object, class ConstIterator> */
template<class Object, class ConstIterator>
inline ConstIterator unserialize(Object& o, ConstIterator i) {
  return o.unserialize(i);
}

/* Return number of bytes needed by this object when serialized. If a
   whole tree of objects is serialized, this may include the
   accumulated serialized sizes of the child objects, too. */
template<class Object>
inline size_t serialSizeOf(const Object& o) {
  return o.serialSizeOf();
}
//______________________________________________________________________

// Serializations of common data types. Stores in little-endian.
//________________________________________

// Numeric types - append the number of bytes to use (e.g. 4 for 32-bit)

template<class NumType, class Iterator>
inline Iterator serialize1(NumType x, Iterator i) {
  *i = x & 0xff; ++i;
  return i;
}
template<class NumType, class ConstIterator>
inline ConstIterator unserialize1(NumType& x, ConstIterator i) {
  x = static_cast<NumType>(*i); ++i;
  return i;
}

template<class NumType, class Iterator>
inline Iterator serialize2(NumType x, Iterator i) {
  *i = x & 0xff; ++i;
  *i = (x >> 8) & 0xff; ++i;
  return i;
}
template<class NumType, class ConstIterator>
inline ConstIterator unserialize2(NumType& x, ConstIterator i) {
  x = static_cast<NumType>(*i); ++i;
  x |= static_cast<NumType>(*i) << 8; ++i;
  return i;
}

template<class NumType, class Iterator>
inline Iterator serialize4(NumType x, Iterator i) {
  *i = x & 0xff; ++i;
  *i = (x >> 8) & 0xff; ++i;
  *i = (x >> 16) & 0xff; ++i;
  *i = (x >> 24) & 0xff; ++i;
  return i;
}
template<class NumType, class ConstIterator>
inline ConstIterator unserialize4(NumType& x, ConstIterator i) {
  x = static_cast<NumType>(*i); ++i;
  x |= static_cast<NumType>(*i) << 8; ++i;
  x |= static_cast<NumType>(*i) << 16; ++i;
  x |= static_cast<NumType>(*i) << 24; ++i;
  return i;
}

template<class NumType, class Iterator>
inline Iterator serialize6(NumType x, Iterator i) {
  *i = x & 0xff; ++i;
  *i = (x >> 8) & 0xff; ++i;
  *i = (x >> 16) & 0xff; ++i;
  *i = (x >> 24) & 0xff; ++i;
  *i = (x >> 32) & 0xff; ++i;
  *i = (x >> 40) & 0xff; ++i;
  return i;
}
template<class NumType, class ConstIterator>
inline ConstIterator unserialize6(NumType& x, ConstIterator i) {
  x = static_cast<NumType>(*i); ++i;
  x |= static_cast<NumType>(*i) << 8; ++i;
  x |= static_cast<NumType>(*i) << 16; ++i;
  x |= static_cast<NumType>(*i) << 24; ++i;
  x |= static_cast<NumType>(*i) << 32; ++i;
  x |= static_cast<NumType>(*i) << 40; ++i;
  return i;
}

template<class NumType, class Iterator>
inline Iterator serialize8(NumType x, Iterator i) {
  *i = x & 0xff; ++i;
  *i = (x >> 8) & 0xff; ++i;
  *i = (x >> 16) & 0xff; ++i;
  *i = (x >> 24) & 0xff; ++i;
  *i = (x >> 32) & 0xff; ++i;
  *i = (x >> 40) & 0xff; ++i;
  *i = (x >> 48) & 0xff; ++i;
  *i = (x >> 56) & 0xff; ++i;
  return i;
}
template<class NumType, class ConstIterator>
inline ConstIterator unserialize8(NumType& x, ConstIterator i) {
  x = static_cast<NumType>(*i); ++i;
  x |= static_cast<NumType>(*i) << 8; ++i;
  x |= static_cast<NumType>(*i) << 16; ++i;
  x |= static_cast<NumType>(*i) << 24; ++i;
  x |= static_cast<NumType>(*i) << 32; ++i;
  x |= static_cast<NumType>(*i) << 40; ++i;
  x |= static_cast<NumType>(*i) << 48; ++i;
  x |= static_cast<NumType>(*i) << 56; ++i;
  return i;
}
//______________________________________________________________________

#endif
